/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

/** 
 * This interface is used to get the phonetic value of the input text.
 * It can be used to get corresponding phonetic value for ideographic text.
 * The interface can be retrieved by calling QI to the interface 
 * which implements the phonetic interface.
 */

[scriptable, uuid(BC6EA726-AB56-46b6-A21A-AA7B76D6818F)]

interface nsIPhonetic : nsISupports
{
  /**
   * phonetic get the phonetic value of the input text
   */
  readonly attribute DOMString phonetic;
};
