/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[uuid(7f021f5d-f704-4a29-b166-829595169aaf)]
interface nsIMozNavigatorNetwork : nsISupports
{
  // This is a MozConnection
  readonly attribute nsISupports mozConnection;
};
