/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"
#include "nsIEntropyCollector.idl"

[uuid(485b87a8-5dd7-4b8d-8ea8-dee53201f899)]
interface nsIBufEntropyCollector : nsIEntropyCollector
{
  /**
   * Forward the entropy collected so far to |collector| and then
   * continue forwarding new entropy as it arrives.
   */
  void forwardTo(in nsIEntropyCollector collector);

  /**
   * No longer forward to a (possibly) previously remembered collector.
   * Do buffering again.
   */
  void dontForward();
};
